@@ -13,7 +13,7 @@
   performance for heterogeneity.
 */
 
-/* $Id: arrayobject.c,v 1.68 2004/06/07 21:07:07 teoliphant Exp $ */
+/* $Id: arrayobject.c,v 1.72 2004/11/30 18:49:18 teoliphant Exp $ */
 
 #include "Python.h"
 #include <stddef.h>
@@ -540,7 +540,7 @@ static PyObject * PyArray_Resize(PyArrayObject *self, PyObject *shape) {
 	}
 	self->dimensions = dimptr;
 
-	strptr = (int *)realloc(self->dimensions, new_nd*sizeof(int));
+	strptr = (int *)realloc(self->strides, new_nd*sizeof(int));
 	if (strptr == NULL) {
 	    PyErr_SetString(PyExc_MemoryError, "can't allocate memory for array (array may be corrupted).");
 	    return NULL;
@@ -2441,7 +2441,7 @@ int PyArray_ValidType(int type) {
 
 PyObject *PyArray_Cast(PyArrayObject *mp, int type) {
     PyArrayObject *rp, *tmp;
-	
+
     if (mp->descr->type_num == PyArray_OBJECT) {
 	return PyArray_FromObject((PyObject *)mp, type, mp->nd, mp->nd);
     }
@@ -2450,7 +2450,7 @@ PyObject *PyArray_Cast(PyArrayObject *mp, int type) {
 	
     rp = (PyArrayObject *)PyArray_FromDims(tmp->nd, tmp->dimensions, type);
     mp->descr->cast[(int)rp->descr->type_num](tmp->data, 1, rp->data, 1, SIZE(mp));
-	
+
     Py_DECREF(tmp);
     return (PyObject *)rp;
 }
@@ -2587,7 +2587,7 @@ extern int PyArray_CanCastSafely(int fromtype, int totype) {
     if (fromtype == totype) return 1;
     if (totype == PyArray_OBJECT) return 1;
     if (fromtype == PyArray_OBJECT) return 0;
-	
+       
     switch(fromtype) {
     case PyArray_CHAR:
 	return 0;
@@ -2595,16 +2595,18 @@ extern int PyArray_CanCastSafely(int fromtype, int totype) {
 	return (totype >= PyArray_SHORT);
     case PyArray_SBYTE:
     case PyArray_SHORT:
+        return (totype > fromtype) && (totype != PyArray_USHORT) && (totype != PyArray_UINT);
     case PyArray_USHORT:
 	return (totype > fromtype);
     case PyArray_INT:
+	return (totype >= PyArray_LONG) && (totype != PyArray_FLOAT) && (totype != PyArray_CFLOAT);
+    case PyArray_UINT:
+        return (totype > PyArray_FLOAT);
     case PyArray_LONG:
 	/*Shouldn't allow Longs to be cast to Ints, not safe on 64-bit machines!*/
-	return (totype >= PyArray_INT) && (totype != PyArray_FLOAT); 
-    case PyArray_UINT:
-        return (totype >= PyArray_FLOAT); 
+	return (totype > PyArray_LONG) && (totype != PyArray_FLOAT) && (totype != PyArray_CFLOAT); 
     case PyArray_FLOAT:
-	return (totype > PyArray_FLOAT);
+	return (totype > fromtype);
     case PyArray_DOUBLE:
 	return (totype == PyArray_CDOUBLE);
     case PyArray_CFLOAT:
@@ -2681,7 +2683,7 @@ extern PyObject * PyArray_Reshape(PyArrayObject *self, PyObject *shape) {
 	return NULL;
     }
 	
-    if (PyArray_As1D(&shape, (char **)&dimensions, &n, PyArray_INT) == -1) return NULL;
+    if (PyArray_As1D(&shape, (char **)&dimensions, &n, PyArray_LONG) == -1) return NULL;
 	
     s_known = 1;
     i_unknown = -1;
